import unittest
import numpy as np
from wiserep_api import get_target_property, get_target_class


class TestProperties(unittest.TestCase):
    def test_properties(self):
        target = '2004eo'
        properties_dict = {'type':'SN Ia',
                           'redshift':'0.015718',
                           'host':'NGC6928',
                           'coords':'20:32:54.190 +09:55:42.71',
                           'coords_deg':'308.22579 +9.92853',
                          } 
        for prop, value in properties_dict.items():
            result = get_target_property(target, prop)
            np.testing.assert_string_equal(str(result), value)

        # multiple properties at once
        properties = list(properties_dict.keys())
        values = get_target_property(target, properties)
        expected_props = ['SN Ia', 0.015718, 'NGC6928', 
                          '20:32:54.190 +09:55:42.71', '308.22579 +9.92853']

        assert values==expected_props, "The values retrieved do not match the expected ones" 

class TestClassification(unittest.TestCase):
    def test_classification(self):
        sn_type = get_target_class("2004eo")
        np.testing.assert_string_equal(sn_type, "SN Ia")


if __name__ == "__main__":
    unittest.main()



import os
import pandas as pd
from astropy.io import fits
from wiserep_api.api import get_response, get_target_response


def exclude_include(url, exclude=None, include=None):
    """Either includes or excludes the given URL according to the
    given patterns.

    Note: only one of the parameters ('exclude' or 'include') can be
    given at a time, not both

    Parameters
    ----------
    url : str
        URL string.
    exclude : list, optional
        Patterns to exclude URL.
    include : _type_, optional
        Patterns to include URL.

    Returns
    -------
    skip: bool
        Whether to exclude/include the given URL.
    """
    err_msg = "'exclude' and 'include' cannot be given at the same time!"
    assert exclude is None or include is None, err_msg

    if exclude is not None:
        skip = False
        for pattern in exclude:
            if pattern in url:
                skip = True

    if include is not None:
        skip = True
        for pattern in include:
            if pattern in url:
                skip = False

    if exclude is None and include is None:
        skip = False

    return skip


def download_target_spectra(
    iau_name, file_type=None, exclude=None, include=None, verbose=False
):
    """Downloads the target's spectra from Wiserep.

    Parameters
    ----------
    iau_name: str
        IAU name of the target (e.g. 2020xne).
    file_type: str
        File format: either 'ascii' or 'fits'. By default,
        both formats are downloaded.
    exclude: list, default 'None'
        Files with the given string patterns are excluded.
        Cannot be given together with 'include'.
    include: list, default 'None'
        Files with the given string patterns are inxcluded.
        Cannot be given together with 'exclude'.
    verbose: bool, default 'False'
        If 'True', print some of the extra information.

    Returns
    -------
    target_class: str
        The target's classification. Returns 'Unknown' if not found.
    """
    if os.path.isdir("spectra") is False:
        os.mkdir("spectra")

    assert file_type in [None, "ascii", "fits"], "not a valide file type"

    # target's url
    response = get_target_response(iau_name, verbose)

    # search for spectra URLs
    # ASCII
    txt_urls = []
    split_text = response.text.split("asciifile=https%3A//")
    for st in split_text:
        url = st.split('"')[0]
        if (url not in txt_urls) and ("&amp" not in url) and ("DOCTYPE" not in url):
            txt_urls.append(url)
    if verbose is True:
        print(f"Found {len(txt_urls)} URLs with spectra (ASCII): {txt_urls}")
        
    # FITS
    fits_urls = []
    split_text = response.text.split("https://")
    for st in split_text:
        url = st.split('"')[0]
        if (url not in fits_urls) and (".fits" in url) and ("rel-file" not in url):
            fits_urls.append(url)
    if verbose is True:
        print(f"Found {len(fits_urls)} URLs with spectra (FITS): {fits_urls}")

    # download ASCII spectra
    if file_type == "ascii" or file_type is None:
        for url in txt_urls:
            # skip files
            skip = exclude_include(url, exclude, include)
            if skip is True:
                if verbose is True:
                    print(f"Skipping {url}")
                continue

            # check url
            response = get_response("http://" + url)
            if response is None:
                print(f"Nothing found in {url}")

            # get spectrum
            basename = os.path.basename(url)
            obj_dir = os.path.join("spectra", iau_name)
            if os.path.isdir(obj_dir) is False:
                os.mkdir(obj_dir)
            outfile = os.path.join(obj_dir, basename)

            with open(outfile, "w") as file:
                file.write(response.text)

            if response.text.startswith("BITPIX") or response.text.startswith("SIMPLE"):
                # file includes header
                for i, line in enumerate(response.text.split("\n")):
                    split_line = line.split()
                    try:
                        _ = float(split_line[0])
                        _ = float(split_line[1])
                        skiprows = i
                        break
                    except:
                        continue
            else:
                skiprows = None
            spec_df = pd.read_csv(
                outfile,
                delim_whitespace=True,
                names=["wave", "flux", "flux_err"],
                comment="#",
                skiprows=skiprows,
            )
            spec_df.to_csv(outfile, index=False)

    # download FITS spectra
    if file_type == "fits" or file_type is None:
        for url in fits_urls:
            # skip files
            skip = exclude_include(url, exclude, include)
            if skip is True:
                if verbose is True:
                    print(f"Skipping {url}")
                continue

            # download file
            print(url)
            hdu = fits.open("http://" + url)

            basename = os.path.basename(url)
            obj_dir = os.path.join("spectra", iau_name)
            if os.path.isdir(obj_dir) is False:
                os.mkdir(obj_dir)
            outfile = os.path.join(obj_dir, basename)
            if os.path.isfile(outfile) is True:
                overwrite = True
            else:
                overwrite = False

            hdu.writeto(outfile, overwrite=overwrite, output_verify="ignore")



import os
import glob
import pandas as pd


def create_snid_inputs(directory):
    """Creates input files with the spectra for SNID.

    Parameters
    ----------
    directory : str
        Target's directory, e.g. ``spectra/2004eo``.
    """
    # get every single file for this SN
    all_directory_files = glob.glob(os.path.join(directory, "*"))

    # get only the text files, excluding SNID outputs from previous runs
    sn_files = [
        file
        for file in all_directory_files
        if file.endswith(".fits") is False
        and "snid" not in file
        and "output" not in file
    ]

    # create a new file in the format that SNID likes
    for file in sn_files:
        with open(file, 'r') as f:
            first_line = f.readlines()[0]
            if 'wave' not in first_line or 'flux' not in first_line:
                # this is not a spectrum
                continue
            
        sn_df = pd.read_csv(file)
        snid_df = sn_df[["wave", "flux"]]
        outfile = os.path.splitext(file)[0] + ".snid"
        snid_df.to_csv(outfile, header=False, index=False, sep="\t")


def run_snid(directory, command=None, skip_fits=False):
    """Runs SNID on the given directory.

    SNID is run on all the files with spectra in the given directory.
    For more information, check the SNID website:
    https://people.lam.fr/blondin.stephane/software/snid/howto.html

    Parameters
    ----------
    directory : str
        Target's directory, e.g. ``spectra/2004eo``.
    command : str, optional
        SNID command. By default, ``snid inter=0 plot=0 aband=0`` is used.
    skip_fits: bool, ``False``
        Whether to skip files that were already run with SNID.
    """
    create_snid_inputs(directory)

    if command is None:
        command = "snid inter=0 plot=0 aband=0"

    # get SNID input files
    all_directory_files = glob.glob(os.path.join(directory, "*"))
    snid_files = [file for file in all_directory_files if file.endswith(".snid")]

    for file in snid_files:
        # get output file name
        basename = os.path.basename(file)
        print_file = os.path.splitext(basename)[0]
        snid_output = os.path.join(directory, f"{print_file}_snid.output")
        
        if skip_fits is True and os.path.isfile(snid_output) is True:
            # skip this file that was previously fitted with SNID
            continue

        # go to the SN directory, run SNID and come back
        os.chdir(directory)
        os.system(f"{command} {basename}")
        os.chdir("../..")

        # remove temporary files
        os.remove(file) 
        snid_param_file = os.path.join(directory, "snid.param")
        os.remove(snid_param_file)
        
        # check SNID output file
        if os.path.isfile(snid_output) is False:
            print(f"\nNo SNID output found for {print_file}! Probably a failed fit...")
            
            continue

        # get SNID output and "clean" it
        with open(snid_output) as f:
            skiprows = 0
            for i, line in enumerate(f.readlines()):
                if line.startswith("#no. sn type"):
                    break
                skiprows += 1

        # save new output
        output_df = pd.read_csv(snid_output, skiprows=skiprows, delim_whitespace=True)
        output_df = output_df[output_df.grade == "good"]  # keep only the good fits
        output_df.to_csv(snid_output, index=False)



import os
import glob
import json
import numpy as np
import wiserep_api
from wiserep_api.api import get_response

wiserep_api_path = wiserep_api.__path__[0]

spec_types_file = os.path.join(wiserep_api_path, "static", "spectral_types.json")
with open(spec_types_file, "r") as fp:
    spectral_types = json.load(fp)


def print_spectral_types():
    """Prints the spectral types as defined by Wiserep"""
    global spectral_types
    print(spectral_types)


def download_sn_list(spec_type):
    """Downloads a list of all the targets of a given spectral type.

    The spectral types are as defined by Wiserep. To list then,
    you can use ``print_spectral_types()``.

    Parameters
    ----------
    spec_type : int or str
        Spectral type, e.g. ``SN Ia`` or ``3``.
    """
    if os.path.isdir("wiserep") is False:
        os.mkdir("wiserep")

    global spectral_types
    if isinstance(spec_type, str):
        spec_type = spectral_types[spec_type]

    # search url
    url = f"https://www.wiserep.org/search?&page=0&public=all&type[]={spec_type}"

    spec_directory = os.path.join("wiserep", str(spec_type))
    # create any missing directory
    if os.path.isdir(spec_directory) is False:
        os.mkdir(spec_directory)

    # start download
    for i in range(0, 999):
        # change page
        page = f"page{i}"
        if i > 0:
            current_page = url.split("page=")[1].split("&")[0]
            url = url.replace(f"&page={current_page}", f"&page={i}")

        # get page data
        response = get_response(url)
        split_text = response.text.split('Click to Object page">')

        # get names of the SNe
        names = []
        for st in split_text:
            name = st.split("</a")[0]
            # alternative name
            alt_split = st.split('target="_blak">')
            if len(alt_split) < 2:
                alt_name = None
            else:
                alt_name = alt_split[1].split("</a")[0]

            if len(name) > 20:
                continue  # this is just text
            try:
                # this avoids some annoying lines
                # and uses the alternative name if
                # no IAU name is found
                _ = float(name)
                if alt_name is not None:
                    if "," in alt_name:
                        # Some SNe have 2+ alternative names
                        alt_name = alt_name.split(",")[0]
                    names.append(alt_name)
            except:
                if name.startswith("SN "):
                    # IAU name
                    name = name.replace("SN ", "")
                names.append(name)

        if len(names) == 0:
            # no more SNe found
            break

        # save page data
        outfile = os.path.join("wiserep", str(spec_type), page + ".txt")
        np.savetxt(outfile, np.array(names).T, fmt="%s")

    # save full list
    sne_list = []
    files = glob.glob(os.path.join("wiserep", str(spec_type), "*"))
    for file in files:
        sne_page = np.genfromtxt(file, dtype=str, delimiter="\n")
        sne_list = sne_list + list(sne_page)

    spec_type_str = [
        key for key, value in spectral_types.items() if value == spec_type
    ][0]
    np.savetxt(
        f'{spec_type_str.replace(" ", "")}_wiserep.txt', np.array(sne_list).T, fmt="%s"
    )
    print(f'{len(sne_list)} "{spec_type_str}" objects found!')
    print(f"URL used: {url}")
